#ifndef USE_METAL
uniform sampler2D diffuseMap;
uniform sampler2D lightMap;

#if defined USE_DIFFUSE_LODBIAS
uniform highp float diffuseLodBias;
#endif

uniform lowp float overbright;
uniform lowp float brightBurn;

varying highp   vec2  uv0;	
varying highp   vec2  uv1;

#ifdef USE_CHEAP_COLOR_CORRECTION
	uniform lowp vec3 CC_Gamma;
	uniform lowp vec3 CC_Luminance;
#endif


#if defined(USE_NDIS)
	uniform sampler2D normalMap;
#endif

#if defined(USE_DECALS)
	uniform sampler2D decalMap;
#endif

#if defined(USE_DECALS)
	varying highp   vec2  uv2;
#endif

#if defined(USE_SNOWCOVERED)
	uniform sampler2D snowMap;
	uniform sampler2D edgeMap;
	uniform	mediump float snowThreshold;
	uniform	mediump float snowIntensity;
	varying highp   vec2  uvSnow0;
	varying highp   vec2  uvSnow1;
	varying lowp   	float  snowMask;
#endif

	uniform lowp 	vec4 		inFogColor;
	varying lowp 	float 		fogRatio;

#if defined(USE_EMISSIVENESS) || defined(USE_SPECULARMASK)
	uniform sampler2D maskMap;
#endif

#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
	uniform 		sampler2D 	reflectionMap;
	uniform lowp 	float 		reflectionAmount;
	varying highp 	vec2  		VSN;
#endif

#ifdef COMPUTE_LINEAR_DEPTH
	varying highp vec3 positionWorldSpace;
	uniform highp vec4 inLinearDepthPlane;
#endif

void main()
{					
#if defined USE_DIFFUSE_LODBIAS
	lowp vec4 DF 	= jetAlphaTexture2D(diffuseMap , uv0, diffuseLodBias);
#else
	lowp vec4 DF 	= jetAlphaTexture2D(diffuseMap , uv0);
#endif
	lowp vec4 LM 	= jetAlphaTexture2D(lightMap 	, uv1);
	
	#if defined(USE_DECALS)
		lowp vec4 DECAL 	= texture2D(decalMap 	, uv2);
	#endif	
	
	#if defined(USE_NDIS)
		lowp vec4 NM = texture2D(normalMap , uv0) * 2.0 - 1.0;
	#endif
	
	#if defined(USE_EMISSIVENESS) || defined(USE_SPECULARMASK)
		lowp vec4 MK = texture2D(maskMap, uv0);	
	#endif
	
	#if defined(USE_NDIS)
		lowp vec2 NDISuv = vec2((NM.x + VSN.x),(NM.y - VSN.y));
		lowp vec4 SP = texture2D(reflectionMap, NDISuv.xy) * reflectionAmount;
	#endif
	#if defined(USE_VERTEX_NDIS)
		lowp vec4 SP = texture2D(reflectionMap, VSN) * reflectionAmount;
	#endif
	
	#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
		#if defined(USE_SPECULARMASK) 
			SP = (SP * LM) * MK.r;
		#else 
			// Generate a fake spec mask using diffuse luminance and some "luminance constants"
			lowp vec3 Lum = vec3( 0.2126, 0.7152, 0.0722);
			lowp float AvgLum = dot(Lum, DF.rgb);
			SP = (SP * LM) * AvgLum;
		#endif
	#endif
	
	#if defined(USE_EMISSIVENESS)
		LM = min(LM + MK.g, 1.0);
	#endif

	#if defined(USE_SNOWCOVERED)
		lowp vec4 snow	= texture2D(snowMap, uvSnow0);
		lowp vec4 edge	= texture2D(edgeMap, uvSnow1);
		lowp float mixAmount	= min(pow((edge.r * snowMask * snowIntensity), snowThreshold), 1.0);
		DF						= mix(DF, snow, mixAmount);
	#endif
	
	//COMPLETE
	//lowp vec4 Complete = DF * LM + (LM * LM.a * brightBurn) * overbright;
	lowp vec4 Complete = DF * (LM + (LM * LM.a * brightBurn)) * overbright;
	
	#if defined(USE_DECALS)
		Complete *= DECAL;
	#endif	
	
	#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
		Complete += SP;
	#endif
	
	Complete = mix(Complete, inFogColor, fogRatio);
	
	#ifdef COMPUTE_LINEAR_DEPTH
		#define gl_FragColor gl_FragData[0]
	#endif
	
	gl_FragColor = vec4(Complete.rgb,DF.a);

	#if defined(USE_CHEAP_COLOR_CORRECTION)
		lowp vec3 _fragColor_ = gl_FragColor.rgb;
		gl_FragColor.rgb = (_fragColor_.rgb * CC_Luminance) + ((1.0 - _fragColor_.rgb) * CC_Gamma);
	#endif
	
	#if defined(COMPUTE_LINEAR_DEPTH)
		gl_FragData[1] = vec4(dot(vec4(positionWorldSpace.xyz, 1.0), inLinearDepthPlane));
	#endif
	
	DEBUG_CHECK_UV(uv0);
	DEBUG_CHECK_UV(uv1);
}

#else

// metal code
void fs()
{
	PixelOutput out;
#if defined USE_DIFFUSE_LODBIAS
	half4 DF 	= (half4)diffuseMap.sample(sampler_diffuseMap, in.uv0, bias(cb.diffuseLodBias));
#else
	half4 DF 	= (half4)diffuseMap.sample(sampler_diffuseMap, in.uv0);
#endif
	half4 LM 	= (half4)lightMap.sample(sampler_lightMap, in.uv1);
	
	#if defined(USE_DECALS)
		half4 DECAL 	= (half4)decalMap.sample(sampler_decalMap, in.uv2);
	#endif	
	
	#if defined(USE_NDIS)
		half4 NM = (half4)normalMap.sample(sampler_normalMap, in.uv0) * 2.0 - 1.0;
	#endif
	
	#if defined(USE_EMISSIVENESS) || defined(USE_SPECULARMASK)
		half4 MK = (half4)maskMap.sample(sampler_maskMap, in.uv0);	
	#endif
	
	#if defined(USE_NDIS)
		float2 NDISuv = float2((NM.x + in.VSN.x),(NM.y - in.VSN.y));
		half4 SP = half4(reflectionMap.sample(sampler_reflectionMap, NDISuv.xy) * cb.reflectionAmount);
	#endif
	#if defined(USE_VERTEX_NDIS)
		half4  SP = half4(reflectionMap.sample(sampler_reflectionMap, in.VSN) * cb.reflectionAmount);
	#endif
	
	#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
		#if defined(USE_SPECULARMASK) 
			SP = (SP * LM) * MK.r;
		#else 
			// Generate a fake spec mask using diffuse luminance and some "luminance constants"
			half3 Lum = half3( 0.2126f, 0.7152f, 0.0722f);
			half AvgLum = dot(Lum, DF.rgb);
			SP = (SP * LM) * AvgLum;
		#endif
	#endif
	
	#if defined(USE_EMISSIVENESS)
		LM = min(LM + MK.g, 1.0h);
	#endif

	#if defined(USE_SNOWCOVERED)
		half4 snow	= (half4)snowMap.sample(sampler_snowMap, in.uvSnow0);
		half4 edge	= (half4)edgeMap.sample(sampler_edgeMap, in.uvSnow1);
		half mixAmount	= min((half)pow((edge.r * in.snowMask * (half)cb.snowIntensity), (half)cb.snowThreshold), 1.0h);
		DF						= mix(DF, snow, mixAmount);
	#endif
	
	//COMPLETE
	half4 Complete = DF * (LM + (LM * LM.a * (half)cb.brightBurn)) * (half)cb.overbright;
	
	#if defined(USE_DECALS)
		Complete *= DECAL;
	#endif	
	
	#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
		Complete += SP;
	#endif
	
	Complete = mix(Complete, (half4)cb.inFogColor, in.fogRatio);
		
	half4 fragColor = half4(Complete.rgb,DF.a);

	#if defined(USE_CHEAP_COLOR_CORRECTION)
		fragColor.rgb = (fragColor.rgb * CC_Luminance) + ((1.0 - fragColor.rgb) * cb.CC_Gamma);
	#endif
	out.color = fragColor;
	
	#if defined(COMPUTE_LINEAR_DEPTH)
		out.color1 = float4(dot(float4(in.positionWorldSpace.xyz, 1.0), cb.inLinearDepthPlane));
	#endif
	return out;
}

#endif